% Solve a linear rational expectations model with AIM.  Check the 
% accuracy of the solution, display the roots of the system, and
% compute the observable structure.

% make sure to run SETPATHGARY to include aim programs on search path

printflag=1;

% import cof matrix
neq = 134;
nlag = 4;
nlead = 3;
cof = load('h:\trmaintsparse\hmat.txt');
cof = reshape(cof,neq,neq*(nlag+nlead+1));
nperiods = 10;      % number of periods for IRFs
nshock=113;         % position in state space of variable to shock
sizeshock=0.01;     % size of shock
printflag = 1;


% Numerical tolerances for aim

epsi   = 2.2e-16;
condn  = 1.e-10;
uprbnd = 1 + 1.e-6;

% ---------------------------------------------------------------------
% Construct structural coefficient matrix.
% ---------------------------------------------------------------------


t0 = clock;
[q,cofb,rts,ia,nex,nnum,lgrts,aimcode] = SPAmalg(cof,neq,nlag,nlead,condn,uprbnd);

if aimcode>1,
  disp(SPAimerr(aimcode));
else
    if (printflag)

        disp(SPAimerr(aimcode));
	 
	    disp(['Elapsed time                         = ',num2str(etime(clock,t0))]);
	    disp(['Number of exact shiftrights (nex)    = ',num2str(nex)]);
	    disp(['Number of numeric shiftrights (nnum) = ',num2str(nnum)]);
	    disp(['Number of large roots (lgrts)        = ',num2str(lgrts)]);
	    disp(['(nex + nnum + lgrts) - neq*nlead     = ',num2str(nex+nnum+lgrts-neq*nlead)]);
	    disp(['Dimension of companion matrix (ia)   = ',num2str(ia)]);
    end
end

size(cofb)
% ---------------------------------------------------------------------
% Display roots, magnitude of roots and period
% ---------------------------------------------------------------------

if (printflag)
   pcvibes(rts,0);
end

% ---------------------------------------------------------------------
% generate impulse response functions
% ---------------------------------------------------------------------

history = zeros(neq,nlag);

% put in a government consumption shock
history(nshock,nlag)=sizeshock;

for dt=nlag+1:nlag+nperiods
    state = zeros(neq*nlag,1);
    for stateindx = 1:nlag
        state(1+(stateindx-1)*neq:stateindx*neq)=history(:,dt-nlag+stateindx-1);
    end
    newstate = cofb*state;
    history = [history, newstate];
end

cofbr = reshape(cofb,neq*neq*nlag,1);

save h:\trmaintsparse\cofbMatlab.txt -ascii -double cofbr