% generate impulse response functions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%            Assign values to variables below:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

neq =           % number of equations (matches number of endogenous variables)
nlag =          % maximum lag in the system of equations
nperiods =      % number of periods for IRFs
sizeshock =     % size of shock
nshock =        % position in the state vector of variable to shock


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% initialize predetermined values to 0
history = zeros(neq,nlag);

cofb = cofb(1:neq,:)    % redefines cofb to be the partition of cofb that 
                        % yields a decision rule for the endogenous
                        % variables based on their predetermined values

% put in a shock on the nshock variable
% to shock more than one variable add extra nonzero values for the 
% predetermined space following the model below
history(nshock,nlag)=sizeshock;

% create IRFs
% history will hold the IRFs for all the variables in the state space
for dt=nlag+1:nlag+nperiods
    state = zeros(neq*nlag,1);
    for stateindx = 1:nlag
        state(1+(stateindx-1)*neq:stateindx*neq)=history(:,dt-nlag+stateindx-1);
    end
    newstate = cofb*state;
    history = [history, newstate];
end

