/*

  PARSE1.C	.MEX file corresponding to do first part of parsing; i.e.
		run routine that defines endog_, param_, eqtype_, vtype_,
                delay_, neq, nlag, nlead, etc.

  The calling syntax is:

[param,np,modname,neq,nlag,nlead,eqname,eqtype,endog,delay,vtype]=parse1;


*/

#include <math.h>
#include "mex.h"

void mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
mxArray *plhs[], *prhs[];
/* Matlab version update: Type of *plhs[] and *prhs[] was changed from Matrix to mxArray */
{           /* Type local variables */
  int i;
  int j;
  int np;
  int neq, nlag, nlead;
  int **eqtype = (int **) malloc(sizeof(int*));
  int **delay = (int **) malloc(sizeof(int*));
  int **vtype = (int **) malloc(sizeof(int*));
  char **modname = (char **) malloc(sizeof(char*));
  char ***param = (char ***) malloc(sizeof(char**));
  char ***eqname = (char ***) malloc(sizeof(char**));
  char ***endog = (char ***) malloc(sizeof(char**));

  double *prparam, *prnp,  *prmodname,  *prneq,  *prnlag,  *prnlead;  
  double *preqname,  *preqtype,  *prendog,  *prdelay,  *prvtype;  


	/* Check for proper number of arguments */

	if (nrhs != 0) {
		mexErrMsgTxt("PARSE1 takes no input arguments.");
	} else if (nlhs != 11) {
		mexErrMsgTxt("PARSE1 returns eleven output arguments.");
	}


	/* Call compute_aim to get data locally */

       compute_aim_data(param, &np, modname, &neq, &nlag, &nlead, 
	                eqname, eqtype, endog, delay, vtype);

	/* Create matrices for the return arguments */
	/* Matlab version update: mxCreateFull was replaced by mxCreateDoubleMatrix
		and type for third argument changed from REAL to mxREAL */

	 plhs[0]  = mxCreateDoubleMatrix(np, 8, mxREAL);
	 plhs[1]  = mxCreateDoubleMatrix(1, 1, mxREAL);
	 plhs[2]  = mxCreateDoubleMatrix(13, 1, mxREAL);
	 plhs[3]  = mxCreateDoubleMatrix(1, 1, mxREAL);
	 plhs[4]  = mxCreateDoubleMatrix(1, 1, mxREAL);
	 plhs[5]  = mxCreateDoubleMatrix(1, 1, mxREAL);
	 plhs[6]  = mxCreateDoubleMatrix(neq, 8, mxREAL);
	 plhs[7]  = mxCreateDoubleMatrix(neq, 1, mxREAL);
	 plhs[8]  = mxCreateDoubleMatrix(neq, 8, mxREAL);
	 plhs[9]  = mxCreateDoubleMatrix(neq, 1, mxREAL);
	 plhs[10] = mxCreateDoubleMatrix(neq, 1, mxREAL);

       /* Dereference pointers  */

	  prparam   = mxGetPr(plhs[0]);
	  prnp      = mxGetPr(plhs[1]);
	  prmodname = mxGetPr(plhs[2]);
	  prneq     = mxGetPr(plhs[3]);
	  prnlag    = mxGetPr(plhs[4]);
	  prnlead   = mxGetPr(plhs[5]);
	  preqname  = mxGetPr(plhs[6]);
	  preqtype  = mxGetPr(plhs[7]);
	  prendog   = mxGetPr(plhs[8]);
	  prdelay   = mxGetPr(plhs[9]);
	  prvtype   = mxGetPr(plhs[10]);

       /* Assign the local variables returned from the function
          call to the pointers.  Here, we convert integers np, 
          neq, nlag, nlead to real 1 x 1 matrices.  We convert
          the integer arrays eqtype, delay and vtype to be real
          arrays.  We finally convert the character arrays param, 
          modname, eqname and endog to ascii numeric real arrays. */

          /* Integers */
         
          *prnp     = np;
          *prneq    = neq;
          *prnlag   = nlag;
          *prnlead  = nlead;

          /* Integer arrays to reals */

          for (i = 0; i < neq; i++)
             { preqtype[i] = (double)(*eqtype)[i];
               prvtype[i]  = (double)(*vtype)[i];
               prdelay[i]  = (double)(*delay)[i]; }

          /* Character arrays to ASCII numeric real arrays */

          for (i = 0; i < np; i++)
             { j = 0;
               while ((*param)[i][j] != 0)
                       (prparam)[i+j*np] = (double)(*param)[i][j++];  }
          
          for (i = 0; i < neq; i++)
             { j = 0;
               while ((*endog)[i][j] != 0)
                       (prendog)[i+j*neq] = (double)(*endog)[i][j++];  }

          for (i = 0; i < neq; i++)
             { j = 0;
               while ((*eqname)[i][j] != 0)
                       (preqname)[i+j*neq] = (double)(*eqname)[i][j++];  }

          i = 0;
          while ((i<12) && ((*modname)[i] != 0))
	                prmodname[i] = (double)(*modname)[i++]; 

}
