% Based on /mq/home/aim/utility/solve.m
% Edited for use with PC AIM - CM 6/98
% This is the version of solve that is found in the UNIX aim procs.
% Many features are not supported in the PC version, but this can be
% used to get ideas for ones own solve program.

%%tinit1 = cputime;

% Solve a linear rational expectations model with AIM.  Check the 
% accuracy of the solution, display the roots of the system, and
% compute the observable structure.
% Uses Matlab version of parser

% Set parser flags
% PC version of aim only supports modelez
%%modelez  = 1;
%%economod = 0;

% Request model and parameter file information.
% One can enter this information directly at the command
% line or in a script.

%%dirnam = input('Directory: ');
%%if(isempty(dirnam))
%%  if(exist('olddirnam'))
%%      dirnam = olddirnam;
%%  else
%%      error('No directory name currently defined.')
%%  end
%%end
%%
%%modnam = input('Model file: ');
%%if(isempty(modnam))
%%  if(exist('oldmodnam'))
%%      modnam = oldmodnam;
%%  else
%%      error('No model name currently defined.')
%%  end
%%end
%%
%%parnam = input('Parameter file: ');
%%if(isempty(parnam))
%%  if(exist('oldparnam'))
%%      parnam = oldparnam;
%%  else
%%      error('No parameter program name currently defined.')
%%  end
%%end

%  Request printflag
%%printflag = input('Display diagnostics? ');
%%if(isempty(printflag))
%%   printflag = 0;
%%end


%%directory = ['/mq/home/unsp/aim/matlab/models/',dirnam,'/'];
directory = ['C:\',dirnam,'\'];
eval(['chdir ',directory])

olddirnam = dirnam;
oldmodnam = modnam;
oldparnam = parnam;

%%if (printflag)
%%   space
%%   disp('__________________________________________________________')
%%   space
%%   disp(['Model file:      ',modnam])
%%   disp(['Parameter file:  ',parnam,'.m'])
%%   disp(['Model directory: ',directory])
%%   disp('__________________________________________________________')
%%end

%  Parse model

% Request parseflag
%%parseflag = input('Parse? ');
%%if(isempty(parseflag))
%%   parseflag = 0;
%%end

%  Modelez syntax parser:
%%if(modelez)  myparsem;end
pcparsem;

%  Economod syntax parser: - not supported
%%if(economod) parseem;end 


%  Define the parameter vector.
%  Only need to assign p if you are using pcparsec or tabulator

if( length(param_) )
   eval(parnam)
   [npar,ncols] = size(param_);
   %%p = zeros(npar,1);
   %%for i = 1:npar
   %%   p(i) = eval(param_(i,:));end
%%else
%%   p = [];
end


% Numerical tolerances for aim

%%epsi   = 2.2e-16;
%%condn  = 1.e-8;
%%uprbnd = 1 + 1.e-6;

epsi   = 2.2e-16;
condn  = 1.e-10;
uprbnd = 1 + 1.e-6;

%%if (printflag)
%%   space;
%%   disp('Numerical Tolerances:');
%%   space
%%   disp([' epsi       = ',num2str(epsi)]);
%%   disp([' condn      = ',num2str(condn)]);
%%   disp([' uprbnd - 1 = ',num2str(uprbnd-1)]);
%%   space;
%%end

%%if (printflag & length(param_) )
%%   space(2)
%%   disp('Parameter values:')
%%   space
%%   disp('Name          Value')
%%   tabulator(param_,p);
%%   space(2)
%%end

% ---------------------------------------------------------------------
% Construct structural coefficient matrix.
% ---------------------------------------------------------------------

%  Run compute_aim_matrices.
%  When using pcparsem run compute_aim_matrices directly as a script.
%%tinit0 = cputime

%%[cofg,cofh] = eval([modnam,'2(p,np,neq,nlag,nlead)']);
eval([modnam,'_aim_matrices']);

% Construct cof matrix from cofg, cofh

[rh,ch] = size(cofh);
[rg,cg] = size(cofg);
cof = zeros(rh,ch);
cof(1:rg,1:cg) = cofg;
cof = cof + cofh;

%%disp(['Elapsed time for compute_aim-matrices    = ',num2str(cputime-tinit0)]);

% ---------------------------------------------------------------------
% Run AIM
% ---------------------------------------------------------------------

flops(0);

%%tinit0 = cputime;


[cofb,rts,ia,nex,nnum,lgrts,mcode] = ...
       pcaim_eig(cof,neq,nlag,nlead,condn,uprbnd);

%  These are not supported right now in PC aim 

%% [cofb,rts,rttype,ia,nex,nnum,lgrts,mcode] = ...
%%        pcaim_schur(cof,neq,nlag,nlead,epsi,condn,uprbnd);

%% [cofb,rts,rttype,ia,nex,nnum,lgrts,mcode] = ...
%%        pcaimsub(cof,neq,nlag,nlead,epsi,condn,uprbnd);

if (printflag & mcode>1),
  disp(aimerr(mcode));
end

%%if (printflag)
%%   space
%%   disp(['Elapsed time                         = ',num2str(etime(clock,t0))]);
%%   disp(['Flops                                = ',num2str(flops)]);
%%   space
%%   disp(['Number of exact shiftrights (nex)    = ',num2str(nex)]);
%%   disp(['Number of numeric shiftrights (nnum) = ',num2str(nnum)]);
%%   disp(['Number of large roots (lgrts)        = ',num2str(lgrts)]);
%%   disp(['(nex + nnum + lgrts) - neq*nlead     = ',num2str(nex+nnum+lgrts-neq*nlead)]);
%%   disp(['Dimension of companion matrix (ia)   = ',num2str(ia)]);
%%   disp(['Elapsed time for aim-eig    = ',num2str(cputime-tinit0)]);
%%end

% ---------------------------------------------------------------------
% Display roots, magnitude of roots and period
% ---------------------------------------------------------------------

%%if (printflag)
%%   pcvibes(rts,0);
%%end
% ---------------------------------------------------------------------
% Check accuracy of solution
% ---------------------------------------------------------------------

%%if (printflag)
%%   err = pccheckaim(neq,nlag,nlead,cof,cofb);
%%end

% ---------------------------------------------------------------------
% Compute observable structure
% ---------------------------------------------------------------------

%%tinit0 = cputime;


scof = pcobstruct(cof,cofb,neq,nlag,nlead);

%%disp(['Elapsed time for obstruct    = ',num2str(cputime-tinit0)]);

% ---------------------------------------------------------------------
% Display roots, magnitude of roots and period
% ---------------------------------------------------------------------

%%if (printflag)
%%   pcvibes(rts,0);
%%end
% The End


%%disp(['Elapsed time for it all                     = ',num2str(cputime-tinit1)])
