function add_potential(endog_, param_, param_exceptions, clist)
% endog_ is the list of endogenous variables
% param_ is the list of parameters
% param exceptions contains the names of the parameters that
% need to take a different value in the potential model (all the parameters
% for the nominal rigidities)
% clist is the list of countries

global EQUATIONLIST

delimiters = char(',',';','(',')','+','-','^','*','/',' ','>','=');
numb3rs = char('1','2','3','4','5','6','7','8','9','0','.');


% find the innovations to the shock processes
% assume any variable with err in its name is an innovation
% the innovations are not replicated in the potential block
% and need to be excluded from the list of endogenous variables
nendog = size(endog_,1);
indxerr = 0;
errposlist = [];
for indxvar = 1:nendog
    
    if ~isempty(strfind(endog_(indxvar,:),'err'))
        indxerr = indxerr+1;
        errposlist(indxerr)=indxvar;
    end
end


% remove the param_exceptions from the list of parameters
% the parameters in the list param_exceptions will have _pot appended to them 
exceptionlist = [];
ncountries = size(clist,1);
nparam_exceptions = size(param_exceptions,1);
indxx=0;
for indxc=1:ncountries
    indxx=indxx+1;
    for indxp = 1:nparam_exceptions
        exception = [clist(indxc,:),param_exceptions(indxp,:)];
        exceptionlist = [exceptionlist,strmatch(exception,param_,'exact')]; 
    end
end


nparam = size(param_,1);
indxj =0;
paramposlist = [];
for indxi=1:nparam
    if isempty(find(indxi==exceptionlist))
        indxj=indxj+1;
        paramposlist(indxj) = indxi;
    end
end

% create taboo list, parameters and innovations that are not going to be 
% specific to the potential model
newparam_= param_(paramposlist,:);
exog_ = endog_(errposlist,:);

taboolist_cellstr = cellstr(strvcat(newparam_,exog_,delimiters,char('LEAD','LAG')));

% exclude the innovations to the shock processes from the list of variables
endogposlist = [];
indxj =0;
for indxi=1:nendog
    if isempty(find(indxi==errposlist))
        indxj=indxj+1;
        endogposlist(indxj) = indxi;
    end
end
newendog_ = endog_(endogposlist,:);

% loop through newendog to add potential equations
nnewendog = size(newendog_,1);
for  indxvar = 1:nnewendog
    display(['Processing equation ',num2str(indxvar),' out of ',num2str(nnewendog)])
    
    % skip first part containing symbols for the aim parser
    equation = EQUATIONLIST(endogposlist(indxvar),6:end);
    tokens = tokenize(equation,delimiters);
    ntokens = length(tokens);

    % go through tokens, if token is not a delimiter, append suffix.
    for itok = 1:ntokens
        tidbit = char(deblank(tokens(itok)));
        if isempty(strmatch(tidbit,taboolist_cellstr,'exact'))
                if isempty(strmatch(tidbit(1),numb3rs)) 
                    tokens(itok) = cellstr([tidbit,'_pot']);
                end
        end
    end
    
    % merge back the various tokens and the new equation
    equation = strmerge(tokens);
    addeq(equation,[deblank(upper(endog_(endogposlist(indxvar),:))),'_POT'],'_NOTD');
end
