function aim2dynare(modelname,simpleparamfile,compparamfile,equationlist)

%%
% get information about model that was processed by AIM 
eval(['[param_,np,modname,neq,nlag,nlead,eqname_,eqtype_,endog_,delay_,vtype_] = ',...
     modelname,'_aim_data()']);

%%
% split variables in endog_ into endogenous and innovations
% exogenous variable have names ending in err.

% uncomment code below to define exogenous variables as those that end in
% err
%errposlist = strmatch('err',endog_)
% checkederrposlist = [];
% nerrposlist = length(errposlist);
% indxerr=0;
% for indxi=1:nerrposlist
%     varname = endog_(errposlist(indxi),:);
%     ending = varname(end-2:end);
%     if strmatch(ending,'err','exact')
%         indxerr = indxerr+1;
%         checkederrposlist(indxerr) = errposlist(indxi);
%     end
% end

% imposes that all variables that contain the string err are
% innovation terms
nendog = size(endog_,1);
indxerr = 0;
checkderrposlist = [];
for indxvar = 1:nendog
    
    if ~isempty(strfind(endog_(indxvar,:),'err'))
        indxerr = indxerr+1;
        checkederrposlist(indxerr)=indxvar;
    end
end

indxj =0;
for indxi=1:nendog
    if isempty(find(indxi==checkederrposlist))
        indxj=indxj+1;
        endogposlist(indxj) = indxi;
    end
end

exog_ = endog_(checkederrposlist,:);
endog_ = endog_(endogposlist,:);

%%
% start creating cellstr array containing the dynare file
% declare list of endogenous variables

file = strvcat('var',endog_,'; \n');

%%
% declare exogenous variables
file = strvcat(file,'varexo',exog_,'; \n');

%%
% declare parameters
file = strvcat(file,'parameters',param_,'; \n');

%%
% assign parameter values
% paramfile = textread([simpleparamfile,'.m'],'%s', ...
%           'delimiter','\n','whitespace','','bufsize',40000);
% % replace % with \\ in file
% nlines = size(paramfile,1);
% for indxi = 1:nlines
%     paramfile(indxi) = cellstr(strrep(char(paramfile(indxi)),'%','//'));
% end

file = strvcat(file,simpleparamfile,'\n');

%%
% assign values to composite parameters
% paramfile = textread([compparamfile,'.m'],'%s', ...
%           'delimiter','\n','whitespace','','bufsize',40000);
% % replace % with \\ in file
% nlines = size(paramfile,1);
% for indxi = 1:nlines
%     paramfile(indxi) = cellstr(strrep(char(paramfile(indxi)),'%','//'));
% end

file = strvcat(file,compparamfile,'\n');

%% 
% append equations
% but first process them so that LEAD and LAG commands are translated to 
% dynare format

delimiters = char(' ','(',')',',','*','/','+','-','=','^');

nequations = size(equationlist,1);
newequationlist = cell(size(endog_,1),1);
indxj =0;
for indxi = 1:nequations
    if isempty(find(indxi==checkederrposlist))
        indxj = indxj+1;
        equation = equationlist(indxi,:);
        tokens = tokenize(equation,delimiters);
        ntokens = size(tokens,1);
        
        % deal with LEAD
        leadposlist = strmatch('LEAD',tokens,'exact');
        if ~isempty(leadposlist)
            nlead = length(leadposlist);
            for indxk = 1:nlead
                tokens(leadposlist(indxk)) = {''};
                tokens(leadposlist(indxk)+1) = tokens(leadposlist(indxk)+2);
                tokens(leadposlist(indxk)+2) = {'('};
                tokens(leadposlist(indxk)+3) = tokens(leadposlist(indxk)+4);
                tokens(leadposlist(indxk)+4) = {''};
            end
        end
        
        % deal with LAG
        leadposlist = strmatch('LAG',tokens,'exact');
        if ~isempty(leadposlist)
            nlead = length(leadposlist);
            for indxk = 1:nlead
                tokens(leadposlist(indxk)) = {''};
                tokens(leadposlist(indxk)+1) = tokens(leadposlist(indxk)+2);
                tokens(leadposlist(indxk)+2) = {'('};
                tokens(leadposlist(indxk)+3) = cellstr(['-',char(tokens(leadposlist(indxk)+4))]);
                tokens(leadposlist(indxk)+4) = {''};
            end
        end
        newequationlist(indxj) = {[deblank(char(strmerge(tokens(5:end)))),';']};
        
    end
    
end

%% Eliminate lags from error terms
nlines = size(newequationlist,1);

delimiters = char(' ','(',')',',','*','/','+','-','=','^',';');
for indxline = 1:nlines
    line = char(newequationlist(indxline,:));
    if ~isempty(strfind(line,'err'))
        tokens = tokenize(line,delimiters);
        ntokens = length(tokens);
        for indxtok = 1:ntokens
            tidbit = char(tokens(indxtok));
            if ~isempty(strfind(tidbit,'err'))
                tokens(indxtok+1) = {''};
                tokens(indxtok+2) = {''};
                tokens(indxtok+3) = {''};
                tokens(indxtok+4) = {''};
            end
        end
        newline = deblank(char(strmerge(tokens)))
        newequationlist(indxline) = {newline};
    end
end

%% write file to disk
file = strvcat(file,'model(linear); \n',...
               char(newequationlist),...
               'end; \n',...
               'stoch_simul(order=1,nocorr,nomoments,irf=0);');

savetextfile(file,[modelname,'.mod']);