function troll2dynare(modelname,equationlist)
% This function takes as input a model file written in AIM format
% that has already been parsed by AIM
% and generates a Troll model file
% The name of the troll file is modelname.tpp




%%
% get information about model that was processed by AIM 
eval(['[param_,np,modname,neq,nlag,nlead,eqname_,eqtype_,endog_,delay_,vtype_] = ',...
     modelname,'_aim_data();']);


% create header of troll model file
file = char('addfun main;',...
              'procedure main()',...
              'begin; \n',...
              '>> addsym \n'...
              );
%'>> usemod; ',...
%'>> modedit;',...
                     

%%
% declare parameters
file = strvcat(file,...
               '>> coefficient',...
               char(strcat({'>> '},param_)),...
               '>>,');

%%
% declare list of endogenous variables
file = strvcat(file,...
               '>> endogenous',...
               char(strcat({'>> '},endog_)),...   'use a cell array for >> to include a trailing space
               '>>, \n',...
               '>> ; \n');

           
           
%% 
% append equations
% but first process them so that LEAD and LAG commands are translated to 
% troll format

% define list of delimiters used to split equations into tokens
delimiters = char(' ','(',')',',','*','/','+','-','=','^');

nequations = size(equationlist,1);
newequationlist = cell(size(nequations,1),1);

for indxi = 1:nequations
    equation = equationlist(indxi,:);
    tokens = tokenize(equation,delimiters);
    ntokens = size(tokens,1);

    % deal with LEAD
    leadposlist = strmatch('LEAD',tokens,'exact');
    if ~isempty(leadposlist)
        nlead = length(leadposlist);
        for indxk = 1:nlead
            tokens(leadposlist(indxk)) = {''};
            tokens(leadposlist(indxk)+1) = tokens(leadposlist(indxk)+2);
            tokens(leadposlist(indxk)+2) = {'('};
            tokens(leadposlist(indxk)+3) = tokens(leadposlist(indxk)+4);
            tokens(leadposlist(indxk)+4) = {''};
        end
    end

    % deal with LAG
    leadposlist = strmatch('LAG',tokens,'exact');
    if ~isempty(leadposlist)
        nlead = length(leadposlist);
        for indxk = 1:nlead
            tokens(leadposlist(indxk)) = {''};
            tokens(leadposlist(indxk)+1) = tokens(leadposlist(indxk)+2);
            tokens(leadposlist(indxk)+2) = {'('};
            tokens(leadposlist(indxk)+3) = cellstr(['-',char(tokens(leadposlist(indxk)+4))]);
            tokens(leadposlist(indxk)+4) = {''};
        end
    end
    newequation = ['>> ',endog_(indxi,:),': ',deblank(char(strmerge(tokens(5:end)))),','];
    
    % deal with symbol for exponentiation: translate ^ to **
    newequation = strrep(newequation,'^','**');
    
    % record newequation in list
    newequationlist(indxi) = {newequation};



end



%% write file to disk
file = strvcat(file,'>> addeq bottom ',...
               char(newequationlist),...
               '>> ; \n',...
               'end;'...
               );
%['>> filemod ',modelname,';'],...

%trollmodelname = [modelname,'.tpp'];
savetextfile(file,'bldmod.tpp');
display(['Troll file saved as bldmod.tpp'])