function [paramlist]=replicate_defparamfile(paramfilename,paramlist,clist)
% makes copies of the file whose name is given 
% for each of the countries in the model
% 
% the prefixes in clist are used to create
% country-specific parameters in each file
%
% the parameters are meant to be edited after the replication occurs

ncountries = size(clist,1);

rawfile = textread([paramfilename,'.m'],'%s', ...
          'delimiter','\n','whitespace','','bufsize',40000);
 
nlines = size(rawfile,1);

delimiters = char(',',';','(',')','+','-','^','*','/',' ','>','=');


% loop through countries
for indxc = 1:ncountries
   home = clist(indxc,:);
   
   % open output file for writing
   fid = fopen([paramfilename,'_',home,'.m'],'w+');

   % loop through lines in parameter file
   for numline=1:nlines
        line = char(rawfile(numline));
        % process line if it contains an equal sign
        % otherwise dump it back as is
        if ~isempty(strfind(line,'='))
            % make sure the equal sign is outside of a comment
            posequal = min(strfind(line,'='));
            poscomment = strfind(line,'%');
            if isempty(poscomment)
                poscomment = 1e15;
            end
            if posequal<poscomment
            % split line into tokens based on equal sign
            tokens = tokenize(line,delimiters);
            if (indxc==1)
                paramlist = strvcat(paramlist,char(deblank(tokens(1))));
            end
            tokens(1) = cellstr([home,char(deblank(tokens(1)))]);
            ntokens = length(tokens);
            for indxtoken = 3:ntokens
                tidbit = char(deblank(tokens(indxtoken)));
                if ~isempty(strmatch(tidbit,paramlist,'exact'))
                    tokens(indxtoken) = cellstr([home,tidbit]);
                end
            end
            line = strmerge(tokens);
            
            end
        end
        % the % character is special
        % to print properly to file, replace it with %%
        if ~isempty(strfind(line,'%'));
            line = strrep(line,'%','%%');
        end
        fprintf( fid, [line,'\n'] );
   end
   
   % close output file
   fclose(fid);
end
